use clap::{arg, Command};
use work_timer_cli::commands::{start, stop, edit, status, Settings};

fn cli() -> Command {
    Command::new("timer")
        .about("A tracker for time spent working")
        .subcommand_required(true)
        .arg_required_else_help(true)
        .allow_external_subcommands(true)
        .arg(
            arg!(--project <PROJECT> "The project associated with this session")
            .short('p')
            .default_value("trackbox")
        )
        .arg(
            arg!(--server <URL> "The base URL of the tracking server")
            .short('s')
            .default_value("http://localhost:3000")
        )
        .arg(
            arg!(--json "Use JSON output")
            .short('j')
            .default_value("true")
            .default_missing_value("false")
        )
        .subcommand(
            Command::new("start")
                .about("Start tracking a working session")
                .arg(
                    arg!(<DESCRIPTION> "A description to add for this session")
                    .required(false)
                )
        )
        .subcommand(
            Command::new("stop")
                .about("Finish the working session currently being tracked")
                .arg(
                    arg!(<ID> "The ID of the session to stop")
                    .required(false)
                )
        )
        .subcommand(
            Command::new("edit")
                .about("Edit a list of sessions")
                .arg(
                    arg!(-n <NUM> "The maximum number of sessions to edit")
                    .default_value("10")
                    .required(false)
                )
                .arg(
                    arg!(--since <TIMESTAMP> "A timestamp to start from. Can be ISO8601 or 'today' or a weekday")
                    .required(false)
                )
                .arg(
                    arg!(--until <TIMESTAMP> "A timestamp to end at. Can be ISO8601 or 'today' or a weekday")
                    .required(false)
                    )
        )
        .subcommand(
            Command::new("status")
                .about("Get an overview of recent sessions")
        )
}

#[tokio::main]
async fn main() {
    let matches = cli().get_matches();
    let project = matches.get_one::<String>("project").unwrap();
    let url = matches.get_one::<String>("server").unwrap();
    let json = matches.get_one::<bool>("json").unwrap();
    let settings = Settings { project: project.to_string(), url: url.to_string(), json: *json };

    match matches.subcommand() {
        Some(("start", sub_matches)) => {
            let description = sub_matches.get_one::<String>("DESCRIPTION");
            start(settings, description).await.unwrap();
        }
        Some(("stop", sub_matches)) => {
            let id = sub_matches.get_one::<i32>("ID");
            stop(settings, id).await.unwrap();
        }
        Some(("edit", sub_matches)) => {
            let since = sub_matches.get_one::<String>("since");
            let until = sub_matches.get_one::<String>("until");
            let num = sub_matches.get_one::<String>("NUM");
            edit(settings, since, until, num).await.unwrap();
        }
        Some(("status", _)) => {
            status(settings).await.unwrap();
        }
        _ => unreachable!(), // If all subcommands are defined above, anything else is unreachable!()
    }
}
