use std::{collections::HashMap, io::Write};
use chrono::Datelike;
use chrono::NaiveDateTime;
use reqwest::Client;
use serde_json::json;
use serde::{Deserialize, Serialize};
use std::{
    env::{temp_dir, var},
    fs::File,
    io::Read,
    process::Command,
};

pub struct Settings {
    pub url: String,
    pub project: String,
    pub json: bool,
}

#[derive(Debug, Serialize, Deserialize)]
pub struct WorkPeriod {
    pub id: i32,
    pub project: String,
    pub start_time: NaiveDateTime,
    pub end_time: Option<NaiveDateTime>,
    pub description: Option<String>,
}

pub async fn start(settings: Settings, description: Option<&String>) -> Result<(), reqwest::Error> {
    let mut map = HashMap::new();
    map.insert("project", settings.project);
    if let Some(description) = description {
        map.insert("description", description.to_string());
    }
    let client = Client::new();
    let response = client.post(settings.url + "/api/tracking")
        .json(&map)
        .send()
        .await?;

    if !response.status().is_success() {
        println!("{:?}", response.text().await);
    }
    Ok(())
}

pub async fn stop(settings: Settings, id: Option<&i32>) -> Result<(), reqwest::Error> {
    let mut map = HashMap::new();
    map.insert("project", settings.project);
    if let Some(id) = id {
        map.insert("id", format!("{}", id));
    }

    let client = Client::new();
    let response = client.delete(settings.url + "/api/tracking")
        .json(&map)
        .send()
        .await?;

    if !response.status().is_success() {
        println!("{:?}", response.text().await);
    }
    Ok(())

}

fn parse_timestamp(timestamp: &str) -> String {
    let now = chrono::Local::now();
    let weekdays = ["monday", "tuesday", "wednesday", "thursday", "friday", "saturday", "sunday"];
    if timestamp == "today" {
        return now.format("%Y-%m-%dT00:00:00").to_string();
    } else if timestamp == "yesterday" {
        return (now - chrono::Duration::days(1)).format("%Y-%m-%dT00:00:00").to_string();
    } else {
        for (i, weekday) in weekdays.iter().enumerate() {
            if timestamp == *weekday {
                let days = (now.weekday().num_days_from_monday() as i64 - i as i64) % 7;
                return (now - chrono::Duration::days(days)).format("%Y-%m-%dT00:00:00").to_string();
            }
        }
    }
    return timestamp.to_string();
}

fn parse_periods(body: String) -> Result<Vec<WorkPeriod>, serde_json::Error> {
    let periods: Vec<WorkPeriod> = serde_json::from_str(&body)?;
    Ok(periods)
}

fn to_json(periods: Vec<WorkPeriod>) -> Result<String, serde_json::Error> {
    let json = json!(periods);
    Ok(serde_json::to_string_pretty(&json)?)
}

fn edit_periods(periods: Vec<WorkPeriod>) -> Result<Vec<WorkPeriod>, std::io::Error> {
    let content = to_json(periods)?;
    let editor = var("EDITOR").unwrap_or("vi".to_string());
    let mut file_path = temp_dir();
    file_path.push("Periods.json");
    let mut file = File::create(&file_path).expect("Could not create file");
    file.write_all(content.as_bytes())?;

    Command::new(editor)
        .arg(&file_path)
        .status()
        .expect("Something went wrong");

    let mut editable = String::new();
    File::open(file_path)
        .expect("Could not open file")
        .read_to_string(&mut editable)?;

    let periods: Vec<WorkPeriod> = serde_json::from_str(&editable).expect("Could not parse JSON");
    Ok(periods)
}

pub async fn update_period(settings: &Settings, period: WorkPeriod) -> Result<(), reqwest::Error> {
    let client = Client::new();
    let response = client.put(settings.url.to_string() + "/api/history/" + &period.id.to_string())
        .json(&period)
        .send()
        .await?;

    if !response.status().is_success() {
        println!("{:?}", response.text().await);
    }
    Ok(())
}

pub async fn edit(settings: Settings, since: Option<&String>, until: Option<&String>, num: Option<&String>) -> Result<(), reqwest::Error> {
    let mut params = vec![
        ("project", settings.project.to_owned()),
    ];
    if let Some(since) = since {
        params.push(("since", parse_timestamp(since)));
    }
    if let Some(until) = until {
        params.push(("until", parse_timestamp(until)));
    }
    if let Some(num) = num {
        params.push(("count", num.to_string()));
    }

    let client = Client::new();
    let url = reqwest::Url::parse_with_params((settings.url.to_owned() + "/api/history").as_str(), &params).unwrap();
    let response = client.get(url)
        .send()
        .await?;

    if !response.status().is_success() {
        println!("{:?}", response.text().await);
    } else {
        let body = response.text().await.unwrap();
        let periods = parse_periods(body).unwrap();
        let res = edit_periods(periods).unwrap();
        for period in res {
            update_period(&settings, period).await.unwrap();
        }
    }
    Ok(())
}

pub async fn status(settings: Settings) -> Result<(), reqwest::Error> {
    Ok(())
}
